@@ -84,7 +84,7 @@
 #include "ibConfLex.h"
 
 #define YYERROR_VERBOSE
-static void yyerror(struct YYLTYPE *a, void *b, void *c, const char *s)
+void yyerror(const YYLTYPE *a, void *b, void *c, const char *s)
 {
 	fprintf(stderr, "%s\n", s);
 }
@@ -100,25 +100,27 @@ typedef struct
 	ibBoard_t *boards;
 	unsigned int boards_length;
 	int board_index;
+	int *device_line_numbers;
+	const char *config_file;
 }gpib_yyparse_private_t;
 
-static inline gpib_yyparse_private_t* priv( gpib_yyparse_private_t *parse_arg )
+static inline gpib_yyparse_private_t* priv(gpib_yyparse_private_t *parse_arg)
 {
 	return parse_arg;
 }
 
-static inline ibConf_t* current_config( gpib_yyparse_private_t *parse_arg )
+static inline ibConf_t* current_config(gpib_yyparse_private_t *parse_arg)
 {
 	return &parse_arg->configs[ parse_arg->config_index ];
 }
 
-static inline ibBoard_t* current_board( gpib_yyparse_private_t *parse_arg )
+static inline ibBoard_t* current_board(gpib_yyparse_private_t *parse_arg)
 {
-	if( parse_arg->board_index < 0 ) return NULL;
+	if (parse_arg->board_index < 0) return NULL;
 	return &parse_arg->boards[ parse_arg->board_index ];
 }
 
-void init_gpib_yyparse_private( gpib_yyparse_private_t *priv )
+void init_gpib_yyparse_private(gpib_yyparse_private_t *priv)
 {
 	priv->yyscanner = 0;
 	priv->configs = NULL;
@@ -127,57 +129,104 @@ void init_gpib_yyparse_private( gpib_yyparse_private_t *priv )
 	priv->boards = NULL;
 	priv->boards_length = 0;
 	priv->board_index = -1;
+	priv->device_line_numbers = NULL;
+	priv->config_file = NULL;
 }
 
-int parse_gpib_conf( const char *filename, ibConf_t *configs, unsigned int configs_length,
-	ibBoard_t *boards, unsigned int boards_length )
+static int find_board_config(gpib_yyparse_private_t *priv, int board_index) {
+	int i;
+	for(i = 0; i < priv->configs_length && priv->configs[ i ].defaults.board >= 0; i++) {
+		if (priv->configs[ i ].defaults.board == board_index &&  priv->configs[ i ].is_interface)
+			return i;
+	}
+	return -1;
+}
+
+int parse_gpib_conf(const char *filename, ibConf_t *configs, unsigned int configs_length,
+		    ibBoard_t *boards, unsigned int boards_length, int minor)
 {
 	FILE *infile;
 	int retval = 0;
-	int i;
+	int i,j;
 	gpib_yyparse_private_t priv;
+	int device_line_nos[configs_length];
 
-	if( ( infile = fopen( filename, "r" ) ) == NULL )
-	{
-		fprintf(stderr, "failed to open configuration file\n");
-		setIberr( EDVR );
-		setIbcnt( errno );
+	if ((infile = fopen(filename, "r")) == NULL) {
+		fprintf(stderr, "failed to open configuration file %s\n", filename);
+		setIberr(EDVR);
+		setIbcnt(errno);
 		return -1;
 	}
 
-	init_gpib_yyparse_private( &priv );
+	init_gpib_yyparse_private(&priv);
 	priv.configs = configs;
 	priv.configs_length = configs_length;
 	priv.boards = boards;
 	priv.boards_length = boards_length;
-	for( i = 0; i < priv.configs_length; i++ )
-	{
-		init_ibconf( &priv.configs[ i ] );
-	}
-	for( i = 0; i < priv.boards_length; i++ )
-	{
-		init_ibboard( &priv.boards[ i ] );
+	priv.device_line_numbers = device_line_nos;
+	priv.config_file = filename;
+	for (i=0;i < priv.configs_length;i++)	{
+		init_ibconf(&priv.configs[i]);
+		priv.device_line_numbers[i] = 0;
 	}
+	for (i=0;i < priv.boards_length;i++)
+		init_ibboard(&priv.boards[i]);
 	gpib_yylex_init(&priv.yyscanner);
 	gpib_yyrestart(infile, priv.yyscanner);
-	if(gpib_yyparse(&priv, priv.yyscanner))
-	{
-		fprintf(stderr, "libgpib: failed to parse configuration file\n");
-//XXX setIberr()
-		retval = -1 ;
-	}
+	if (gpib_yyparse(&priv, priv.yyscanner))
+		goto gpib_parse_fail;
 	gpib_yylex_destroy(priv.yyscanner);
 	fclose(infile);
 
-	if( retval == 0 )
-	{
-		for(i = 0; i < priv.configs_length && priv.configs[ i ].defaults.board >= 0; i++)
-		{
-			priv.configs[ i ].settings = priv.configs[ i ].defaults;
+	if (retval == 0) {
+		for (i=0;i<priv.config_index;i++) {
+			priv.configs[i].settings = priv.configs[i].defaults;
+			if (!priv.configs[i].is_interface) {
+				if (!strlen(priv.configs[i].name)) {
+					fprintf(stderr, "No name for device at line %d\n", device_line_nos[i]);
+					goto gpib_parse_fail;
+				}
+				if (priv.configs[i].defaults.board < 0) {
+					fprintf(stderr, "No minor for device \"%s\" at line %d\n", priv.configs[i].name, device_line_nos[i]);
+					goto gpib_parse_fail;
+				}
+				j = find_board_config(&priv, priv.configs[i].defaults.board);
+				if (j < 0) {
+					fprintf(stderr, "Inconsistent config for device \"%s\": no board with minor %d\n",
+						priv.configs[i].name, priv.configs[i].defaults.board);
+					goto gpib_parse_fail;
+				}
+				if (priv.configs[j].defaults.pad < 0) {
+					fprintf(stderr, "No pad configured for minor %d\n", priv.configs[j].defaults.board);
+					goto gpib_parse_fail;
+				}
+				if (priv.configs[i].defaults.pad < 0) {
+					fprintf(stderr, "No pad configured for device \"%s\" at line %i\n",
+						priv.configs[i].name, device_line_nos[i]);
+					goto gpib_parse_fail;
+				}
+				if (priv.configs[i].defaults.pad == priv.configs[j].defaults.pad) {
+					fprintf(stderr, "Address conflict in device  \"%s\" at line %i: has same pad (%d) as board minor %d\n",
+						 priv.configs[i].name, device_line_nos[i], priv.configs[i].defaults.pad, priv.configs[i].defaults.board);
+					goto gpib_parse_fail;
+				}
+			}
+		}
+
+		if (minor >= 0 && find_board_config(&priv,minor) < 0) { // No entry for minor in config file
+			// make one so that udev scripts need not depend on config file
+			i = priv.config_index;
+			priv.configs[i].defaults.board = minor;
+			priv.configs[i].is_interface = 1;
+			snprintf(priv.boards[minor].device, sizeof(priv.boards[minor].device), "/dev/gpib%i", minor);
+			priv.configs[i].settings = priv.configs[i].defaults;
 		}
 	}
-
 	return retval;
+
+gpib_parse_fail:
+	fprintf(stderr, "libgpib: failed to parse configuration file %s\n", filename);
+	return -1;
 }
 
 static void gpib_conf_warn_missing_equals()
@@ -186,7 +235,7 @@ static void gpib_conf_warn_missing_equals()
 }
 
 
-#line 190 "./ibConfYacc.c"
+#line 239 "./ibConfYacc.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -654,12 +703,12 @@ static const yytype_int8 yytranslate[] =
 /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,   142,   142,   143,   144,   145,   152,   163,   173,   174,
-     175,   182,   183,   184,   185,   186,   187,   188,   189,   190,
-     191,   192,   193,   194,   195,   196,   197,   198,   199,   200,
-     205,   210,   215,   222,   233,   234,   235,   243,   244,   245,
-     246,   247,   248,   249,   250,   251,   252,   253,   254,   255,
-     256,   257,   258,   261,   262,   263,   266,   267,   268
+       0,   192,   192,   193,   194,   195,   202,   212,   230,   231,
+     232,   239,   249,   262,   263,   264,   265,   266,   267,   268,
+     269,   270,   271,   272,   273,   274,   275,   276,   277,   278,
+     283,   288,   293,   300,   311,   312,   313,   322,   332,   345,
+     346,   347,   348,   349,   350,   351,   352,   353,   354,   355,
+     356,   357,   358,   361,   362,   363,   366,   367,   368
 };
 #endif
 
@@ -1423,325 +1472,371 @@ yyreduce:
   switch (yyn)
     {
   case 5: /* input: error  */
-#line 146 "ibConfYacc.y"
+#line 196 "ibConfYacc.y"
                         {
-				fprintf(stderr, "input error on line %i of %s\n", gpib_yyget_lineno(priv(parse_arg)->yyscanner), DEFAULT_CONFIG_FILE);
+				fprintf(stderr, "input error on line %i of %s\n", gpib_yyget_lineno(priv(parse_arg)->yyscanner), priv(parse_arg)->config_file);
 				YYABORT;
 			}
-#line 1432 "./ibConfYacc.c"
+#line 1481 "./ibConfYacc.c"
     break;
 
   case 6: /* interface: T_INTERFACE '{' minor parameter '}'  */
-#line 153 "ibConfYacc.y"
+#line 203 "ibConfYacc.y"
                         {
-				current_config( parse_arg )->is_interface = 1;
-				if( ++( priv(parse_arg)->config_index ) >= priv(parse_arg)->configs_length )
-				{
+				current_config(parse_arg)->is_interface = 1;
+				if (++(priv(parse_arg)->config_index) >= priv(parse_arg)->configs_length) {
 					fprintf(stderr, "too many devices in config file\n");
 					YYERROR;
 				}
 			}
-#line 1445 "./ibConfYacc.c"
+#line 1493 "./ibConfYacc.c"
     break;
 
   case 7: /* minor: T_MINOR '=' T_NUMBER  */
-#line 163 "ibConfYacc.y"
+#line 212 "ibConfYacc.y"
                                      {
-				priv(parse_arg)->board_index = (yyvsp[0].ival);
-				current_config(parse_arg)->defaults.board = (yyvsp[0].ival);
-				if(priv(parse_arg)->board_index < priv(parse_arg)->boards_length )
-					snprintf(current_board(parse_arg)->device, sizeof(current_board( parse_arg )->device), "/dev/gpib%i", priv(parse_arg)->board_index);
-				else
+		                int bi = (yyvsp[0].ival);
+
+				if (bi >= 0 && bi < priv(parse_arg)->boards_length) {
+					priv(parse_arg)->board_index = bi;
+					if (strlen(current_board(parse_arg)->device) != 0) {
+						fprintf(stderr, "Duplicate interface minor %d\n", bi);
+						YYERROR;
+					}
+					current_config(parse_arg)->defaults.board = bi;
+					snprintf(current_board(parse_arg)->device, sizeof(current_board(parse_arg)->device), "/dev/gpib%i", bi);
+				} else {
+					fprintf(stderr, "Invalid minor %d\n", bi);
 					YYERROR;
-			}
-#line 1458 "./ibConfYacc.c"
+				}
+                            }
+#line 1514 "./ibConfYacc.c"
     break;
 
   case 10: /* parameter: error  */
-#line 176 "ibConfYacc.y"
+#line 233 "ibConfYacc.y"
                         {
-				fprintf(stderr, "parameter error on line %i of %s\n", (yylsp[0]).first_line, DEFAULT_CONFIG_FILE);
+				fprintf(stderr, "parameter error on line %i of %s\n",gpib_yyget_lineno(priv(parse_arg)->yyscanner)-1, priv(parse_arg)->config_file);
 				YYABORT;
 			}
-#line 1467 "./ibConfYacc.c"
+#line 1523 "./ibConfYacc.c"
     break;
 
   case 11: /* statement: T_PAD '=' T_NUMBER  */
-#line 182 "ibConfYacc.y"
-                                           { current_config( parse_arg )->defaults.pad = (yyvsp[0].ival);}
-#line 1473 "./ibConfYacc.c"
+#line 240 "ibConfYacc.y"
+                {
+			int pad = (yyvsp[0].ival);
+
+			if (pad < 0 || pad > gpib_addr_max) {
+				fprintf(stderr, "Invalid pad %d on line %i in %s\n", pad, gpib_yyget_lineno(priv(parse_arg)->yyscanner),  priv(parse_arg)->config_file);
+				YYABORT;
+			}
+			current_config(parse_arg)->defaults.pad = pad;
+		}
+#line 1537 "./ibConfYacc.c"
     break;
 
   case 12: /* statement: T_SAD '=' T_NUMBER  */
-#line 183 "ibConfYacc.y"
-                                          { current_config( parse_arg )->defaults.sad = (yyvsp[0].ival) - sad_offset;}
-#line 1479 "./ibConfYacc.c"
+#line 250 "ibConfYacc.y"
+                {
+			int sad = (yyvsp[0].ival);
+			if (!sad)
+				sad = -1;
+			else
+				sad -= sad_offset;
+			if (sad < -1 || sad > gpib_sad_max) {
+				fprintf(stderr,"Invalid sad %d on line %i in %s\n", (yyvsp[0].ival), gpib_yyget_lineno(priv(parse_arg)->yyscanner), priv(parse_arg)->config_file);
+				YYABORT;
+			}
+			current_config(parse_arg)->defaults.sad = sad;
+		}
+#line 1554 "./ibConfYacc.c"
     break;
 
   case 13: /* statement: T_EOSBYTE '=' T_NUMBER  */
-#line 184 "ibConfYacc.y"
-                                          { current_config( parse_arg )->defaults.eos = (yyvsp[0].ival);}
-#line 1485 "./ibConfYacc.c"
+#line 262 "ibConfYacc.y"
+                                          { current_config(parse_arg)->defaults.eos = (yyvsp[0].ival);}
+#line 1560 "./ibConfYacc.c"
     break;
 
   case 14: /* statement: T_REOS T_BOOL  */
-#line 185 "ibConfYacc.y"
-                                          { gpib_conf_warn_missing_equals(); current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * REOS;}
-#line 1491 "./ibConfYacc.c"
+#line 263 "ibConfYacc.y"
+                                          { gpib_conf_warn_missing_equals(); current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * REOS;}
+#line 1566 "./ibConfYacc.c"
     break;
 
   case 15: /* statement: T_BIN T_BOOL  */
-#line 186 "ibConfYacc.y"
-                                          { gpib_conf_warn_missing_equals(); current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * BIN;}
-#line 1497 "./ibConfYacc.c"
+#line 264 "ibConfYacc.y"
+                                          { gpib_conf_warn_missing_equals(); current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * BIN;}
+#line 1572 "./ibConfYacc.c"
     break;
 
   case 16: /* statement: T_REOS '=' T_BOOL  */
-#line 187 "ibConfYacc.y"
-                                              { current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * REOS;}
-#line 1503 "./ibConfYacc.c"
+#line 265 "ibConfYacc.y"
+                                              { current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * REOS;}
+#line 1578 "./ibConfYacc.c"
     break;
 
   case 17: /* statement: T_XEOS '=' T_BOOL  */
-#line 188 "ibConfYacc.y"
-                                              { current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * XEOS;}
-#line 1509 "./ibConfYacc.c"
-    break;
-
-  case 18: /* statement: T_BIN '=' T_BOOL  */
-#line 189 "ibConfYacc.y"
-                                             { current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * BIN;}
-#line 1515 "./ibConfYacc.c"
-    break;
-
-  case 19: /* statement: T_EOT '=' T_BOOL  */
-#line 190 "ibConfYacc.y"
-                                             { current_config( parse_arg )->defaults.send_eoi = (yyvsp[0].bval);}
-#line 1521 "./ibConfYacc.c"
-    break;
-
-  case 20: /* statement: T_TIMO '=' T_TIVAL  */
-#line 191 "ibConfYacc.y"
-                                          { current_config( parse_arg )->defaults.usec_timeout = (yyvsp[0].ival); }
-#line 1527 "./ibConfYacc.c"
-    break;
-
-  case 21: /* statement: T_TIMO '=' T_NUMBER  */
-#line 192 "ibConfYacc.y"
-                                           { current_config( parse_arg )->defaults.usec_timeout = timeout_to_usec( (yyvsp[0].ival) ); }
-#line 1533 "./ibConfYacc.c"
-    break;
-
-  case 22: /* statement: T_BASE '=' T_NUMBER  */
-#line 193 "ibConfYacc.y"
-                                          { current_board( parse_arg )->base = (yyvsp[0].ival); }
-#line 1539 "./ibConfYacc.c"
-    break;
-
-  case 23: /* statement: T_IRQ '=' T_NUMBER  */
-#line 194 "ibConfYacc.y"
-                                          { current_board( parse_arg )->irq = (yyvsp[0].ival); }
-#line 1545 "./ibConfYacc.c"
-    break;
-
-  case 24: /* statement: T_DMA '=' T_NUMBER  */
-#line 195 "ibConfYacc.y"
-                                          { current_board( parse_arg )->dma = (yyvsp[0].ival); }
-#line 1551 "./ibConfYacc.c"
-    break;
-
-  case 25: /* statement: T_PCI_BUS '=' T_NUMBER  */
-#line 196 "ibConfYacc.y"
-                                              { current_board( parse_arg )->pci_bus = (yyvsp[0].ival); }
-#line 1557 "./ibConfYacc.c"
-    break;
-
-  case 26: /* statement: T_PCI_SLOT '=' T_NUMBER  */
-#line 197 "ibConfYacc.y"
-                                               { current_board( parse_arg )->pci_slot = (yyvsp[0].ival); }
-#line 1563 "./ibConfYacc.c"
-    break;
-
-  case 27: /* statement: T_MASTER T_BOOL  */
-#line 198 "ibConfYacc.y"
-                                        { gpib_conf_warn_missing_equals(); current_board( parse_arg )->is_system_controller = (yyvsp[0].bval); }
-#line 1569 "./ibConfYacc.c"
-    break;
-
-  case 28: /* statement: T_MASTER '=' T_BOOL  */
-#line 199 "ibConfYacc.y"
-                                        { current_board( parse_arg )->is_system_controller = (yyvsp[0].bval); }
-#line 1575 "./ibConfYacc.c"
-    break;
-
-  case 29: /* statement: T_BOARD_TYPE '=' T_STRING  */
-#line 201 "ibConfYacc.y"
-                        {
-				strncpy(current_board( parse_arg )->board_type, (yyvsp[0].sval),
-					sizeof(current_board( parse_arg )->board_type));
-			}
+#line 266 "ibConfYacc.y"
+                                              { current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * XEOS;}
 #line 1584 "./ibConfYacc.c"
     break;
 
-  case 30: /* statement: T_NAME '=' T_STRING  */
-#line 206 "ibConfYacc.y"
-                        {
-				strncpy(current_config( parse_arg )->name, (yyvsp[0].sval),
-					sizeof(current_config( parse_arg )->name));
-			}
-#line 1593 "./ibConfYacc.c"
+  case 18: /* statement: T_BIN '=' T_BOOL  */
+#line 267 "ibConfYacc.y"
+                                             { current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * BIN;}
+#line 1590 "./ibConfYacc.c"
     break;
 
-  case 31: /* statement: T_SYSFS_DEVICE_PATH '=' T_STRING  */
-#line 211 "ibConfYacc.y"
-                        {
-				strncpy(current_board( parse_arg )->sysfs_device_path, (yyvsp[0].sval),
-					sizeof(current_board( parse_arg )->sysfs_device_path));
-			}
+  case 19: /* statement: T_EOT '=' T_BOOL  */
+#line 268 "ibConfYacc.y"
+                                             { current_config(parse_arg)->defaults.send_eoi = (yyvsp[0].bval);}
+#line 1596 "./ibConfYacc.c"
+    break;
+
+  case 20: /* statement: T_TIMO '=' T_TIVAL  */
+#line 269 "ibConfYacc.y"
+                                          { current_config(parse_arg)->defaults.usec_timeout = (yyvsp[0].ival); }
 #line 1602 "./ibConfYacc.c"
     break;
 
-  case 32: /* statement: T_SERIAL_NUMBER '=' T_STRING  */
-#line 216 "ibConfYacc.y"
+  case 21: /* statement: T_TIMO '=' T_NUMBER  */
+#line 270 "ibConfYacc.y"
+                                           { current_config(parse_arg)->defaults.usec_timeout = timeout_to_usec((yyvsp[0].ival)); }
+#line 1608 "./ibConfYacc.c"
+    break;
+
+  case 22: /* statement: T_BASE '=' T_NUMBER  */
+#line 271 "ibConfYacc.y"
+                                          { current_board(parse_arg)->base = (yyvsp[0].ival); }
+#line 1614 "./ibConfYacc.c"
+    break;
+
+  case 23: /* statement: T_IRQ '=' T_NUMBER  */
+#line 272 "ibConfYacc.y"
+                                          { current_board(parse_arg)->irq = (yyvsp[0].ival); }
+#line 1620 "./ibConfYacc.c"
+    break;
+
+  case 24: /* statement: T_DMA '=' T_NUMBER  */
+#line 273 "ibConfYacc.y"
+                                          { current_board(parse_arg)->dma = (yyvsp[0].ival); }
+#line 1626 "./ibConfYacc.c"
+    break;
+
+  case 25: /* statement: T_PCI_BUS '=' T_NUMBER  */
+#line 274 "ibConfYacc.y"
+                                              { current_board(parse_arg)->pci_bus = (yyvsp[0].ival); }
+#line 1632 "./ibConfYacc.c"
+    break;
+
+  case 26: /* statement: T_PCI_SLOT '=' T_NUMBER  */
+#line 275 "ibConfYacc.y"
+                                               { current_board(parse_arg)->pci_slot = (yyvsp[0].ival); }
+#line 1638 "./ibConfYacc.c"
+    break;
+
+  case 27: /* statement: T_MASTER T_BOOL  */
+#line 276 "ibConfYacc.y"
+                                        { gpib_conf_warn_missing_equals(); current_board(parse_arg)->is_system_controller = (yyvsp[0].bval); }
+#line 1644 "./ibConfYacc.c"
+    break;
+
+  case 28: /* statement: T_MASTER '=' T_BOOL  */
+#line 277 "ibConfYacc.y"
+                                        { current_board(parse_arg)->is_system_controller = (yyvsp[0].bval); }
+#line 1650 "./ibConfYacc.c"
+    break;
+
+  case 29: /* statement: T_BOARD_TYPE '=' T_STRING  */
+#line 279 "ibConfYacc.y"
                         {
-				strncpy(current_board( parse_arg )->serial_number, (yyvsp[0].sval),
-					sizeof(current_board( parse_arg )->serial_number));
+				strncpy(current_board(parse_arg)->board_type, (yyvsp[0].sval),
+					sizeof(current_board(parse_arg)->board_type)-1);
 			}
-#line 1611 "./ibConfYacc.c"
+#line 1659 "./ibConfYacc.c"
+    break;
+
+  case 30: /* statement: T_NAME '=' T_STRING  */
+#line 284 "ibConfYacc.y"
+                        {
+				strncpy(current_config(parse_arg)->name, (yyvsp[0].sval),
+					sizeof(current_config(parse_arg)->name)-1);
+			}
+#line 1668 "./ibConfYacc.c"
+    break;
+
+  case 31: /* statement: T_SYSFS_DEVICE_PATH '=' T_STRING  */
+#line 289 "ibConfYacc.y"
+                        {
+				strncpy(current_board(parse_arg)->sysfs_device_path, (yyvsp[0].sval),
+					sizeof(current_board(parse_arg)->sysfs_device_path)-1);
+			}
+#line 1677 "./ibConfYacc.c"
+    break;
+
+  case 32: /* statement: T_SERIAL_NUMBER '=' T_STRING  */
+#line 294 "ibConfYacc.y"
+                        {
+				strncpy(current_board(parse_arg)->serial_number, (yyvsp[0].sval),
+					sizeof(current_board(parse_arg)->serial_number-1));
+			}
+#line 1686 "./ibConfYacc.c"
     break;
 
   case 33: /* device: T_DEVICE '{' option '}'  */
-#line 223 "ibConfYacc.y"
+#line 301 "ibConfYacc.y"
                         {
-				current_config( parse_arg )->is_interface = 0;
-				if( ++( priv(parse_arg)->config_index ) >= priv(parse_arg)->configs_length )
-				{
+				current_config(parse_arg)->is_interface = 0;
+				priv(parse_arg)->device_line_numbers[priv(parse_arg)->config_index] = gpib_yyget_lineno(priv(parse_arg)->yyscanner);
+				if (++(priv(parse_arg)->config_index) >= priv(parse_arg)->configs_length) {
 					fprintf(stderr, "too many devices in config file\n");
 					YYERROR;
 				}
 			}
-#line 1624 "./ibConfYacc.c"
-    break;
-
-  case 36: /* option: error  */
-#line 236 "ibConfYacc.y"
-                        {
- 				fprintf(stderr, "option error on line %i of config file\n", (yylsp[0]).first_line );
-				YYABORT;
-			}
-#line 1633 "./ibConfYacc.c"
-    break;
-
-  case 37: /* assign: T_PAD '=' T_NUMBER  */
-#line 243 "ibConfYacc.y"
-                                   { current_config( parse_arg )->defaults.pad = (yyvsp[0].ival); }
-#line 1639 "./ibConfYacc.c"
-    break;
-
-  case 38: /* assign: T_SAD '=' T_NUMBER  */
-#line 244 "ibConfYacc.y"
-                                     { current_config( parse_arg )->defaults.sad = (yyvsp[0].ival) - sad_offset; }
-#line 1645 "./ibConfYacc.c"
-    break;
-
-  case 39: /* assign: T_INIT_S '=' T_STRING  */
-#line 245 "ibConfYacc.y"
-                                        { strncpy(current_config( parse_arg )->init_string,(yyvsp[0].sval),60); }
-#line 1651 "./ibConfYacc.c"
-    break;
-
-  case 40: /* assign: T_EOSBYTE '=' T_NUMBER  */
-#line 246 "ibConfYacc.y"
-                                          { current_config( parse_arg )->defaults.eos = (yyvsp[0].ival); }
-#line 1657 "./ibConfYacc.c"
-    break;
-
-  case 41: /* assign: T_REOS T_BOOL  */
-#line 247 "ibConfYacc.y"
-                                          { gpib_conf_warn_missing_equals(); current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * REOS;}
-#line 1663 "./ibConfYacc.c"
-    break;
-
-  case 42: /* assign: T_REOS '=' T_BOOL  */
-#line 248 "ibConfYacc.y"
-                                              { current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * REOS;}
-#line 1669 "./ibConfYacc.c"
-    break;
-
-  case 43: /* assign: T_XEOS '=' T_BOOL  */
-#line 249 "ibConfYacc.y"
-                                              { current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * XEOS;}
-#line 1675 "./ibConfYacc.c"
-    break;
-
-  case 44: /* assign: T_BIN T_BOOL  */
-#line 250 "ibConfYacc.y"
-                                         { gpib_conf_warn_missing_equals(); current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * BIN; }
-#line 1681 "./ibConfYacc.c"
-    break;
-
-  case 45: /* assign: T_BIN '=' T_BOOL  */
-#line 251 "ibConfYacc.y"
-                                             { current_config( parse_arg )->defaults.eos_flags |= (yyvsp[0].bval) * BIN; }
-#line 1687 "./ibConfYacc.c"
-    break;
-
-  case 46: /* assign: T_EOT '=' T_BOOL  */
-#line 252 "ibConfYacc.y"
-                                             { current_config( parse_arg )->defaults.send_eoi = (yyvsp[0].bval);}
-#line 1693 "./ibConfYacc.c"
-    break;
-
-  case 47: /* assign: T_AUTOPOLL  */
-#line 253 "ibConfYacc.y"
-                                          { current_config( parse_arg )->flags |= CN_AUTOPOLL; }
 #line 1699 "./ibConfYacc.c"
     break;
 
-  case 49: /* assign: T_NAME '=' T_STRING  */
-#line 255 "ibConfYacc.y"
-                                        { strncpy(current_config( parse_arg )->name,(yyvsp[0].sval), sizeof(current_config( parse_arg )->name));}
-#line 1705 "./ibConfYacc.c"
+  case 36: /* option: error  */
+#line 314 "ibConfYacc.y"
+                        {
+				int mline =  gpib_yyget_lineno(priv(parse_arg)->yyscanner);
+				fprintf(stderr, "option error on line %i of config file\n", mline);
+				YYABORT;
+			}
+#line 1709 "./ibConfYacc.c"
     break;
 
-  case 50: /* assign: T_MINOR '=' T_NUMBER  */
-#line 256 "ibConfYacc.y"
-                                        { current_config( parse_arg )->defaults.board = (yyvsp[0].ival);}
-#line 1711 "./ibConfYacc.c"
-    break;
+  case 37: /* assign: T_PAD '=' T_NUMBER  */
+#line 323 "ibConfYacc.y"
+                {
+			int pad = (yyvsp[0].ival);
 
-  case 51: /* assign: T_TIMO '=' T_TIVAL  */
-#line 257 "ibConfYacc.y"
-                                          { current_config( parse_arg )->defaults.usec_timeout = (yyvsp[0].ival); }
-#line 1717 "./ibConfYacc.c"
-    break;
-
-  case 52: /* assign: T_TIMO '=' T_NUMBER  */
-#line 258 "ibConfYacc.y"
-                                           { current_config( parse_arg )->defaults.usec_timeout = timeout_to_usec( (yyvsp[0].ival) ); }
+			if (pad < 0 || pad > gpib_addr_max) {
+				fprintf(stderr, "Invalid pad  %d on line %i in %s\n", pad, gpib_yyget_lineno(priv(parse_arg)->yyscanner), priv(parse_arg)->config_file);
+				YYABORT;
+			}
+			current_config(parse_arg)->defaults.pad = pad;
+		}
 #line 1723 "./ibConfYacc.c"
     break;
 
+  case 38: /* assign: T_SAD '=' T_NUMBER  */
+#line 333 "ibConfYacc.y"
+                {
+			int sad = (yyvsp[0].ival);
+			if (!sad)
+				sad = -1;
+			else
+				sad -= sad_offset;
+			if (sad < -1 || sad > gpib_sad_max) {
+				fprintf(stderr, "Invalid sad %d on line %i in %s\n", (yyvsp[0].ival), gpib_yyget_lineno(priv(parse_arg)->yyscanner), priv(parse_arg)->config_file);
+				YYABORT;
+			}
+			current_config(parse_arg)->defaults.sad = sad;
+		}
+#line 1740 "./ibConfYacc.c"
+    break;
+
+  case 39: /* assign: T_INIT_S '=' T_STRING  */
+#line 345 "ibConfYacc.y"
+                                        { strncpy(current_config(parse_arg)->init_string,(yyvsp[0].sval),60); }
+#line 1746 "./ibConfYacc.c"
+    break;
+
+  case 40: /* assign: T_EOSBYTE '=' T_NUMBER  */
+#line 346 "ibConfYacc.y"
+                                          { current_config(parse_arg)->defaults.eos = (yyvsp[0].ival); }
+#line 1752 "./ibConfYacc.c"
+    break;
+
+  case 41: /* assign: T_REOS T_BOOL  */
+#line 347 "ibConfYacc.y"
+                                          { gpib_conf_warn_missing_equals(); current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * REOS;}
+#line 1758 "./ibConfYacc.c"
+    break;
+
+  case 42: /* assign: T_REOS '=' T_BOOL  */
+#line 348 "ibConfYacc.y"
+                                              { current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * REOS;}
+#line 1764 "./ibConfYacc.c"
+    break;
+
+  case 43: /* assign: T_XEOS '=' T_BOOL  */
+#line 349 "ibConfYacc.y"
+                                              { current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * XEOS;}
+#line 1770 "./ibConfYacc.c"
+    break;
+
+  case 44: /* assign: T_BIN T_BOOL  */
+#line 350 "ibConfYacc.y"
+                                         { gpib_conf_warn_missing_equals(); current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * BIN; }
+#line 1776 "./ibConfYacc.c"
+    break;
+
+  case 45: /* assign: T_BIN '=' T_BOOL  */
+#line 351 "ibConfYacc.y"
+                                             { current_config(parse_arg)->defaults.eos_flags |= (yyvsp[0].bval) * BIN; }
+#line 1782 "./ibConfYacc.c"
+    break;
+
+  case 46: /* assign: T_EOT '=' T_BOOL  */
+#line 352 "ibConfYacc.y"
+                                             { current_config(parse_arg)->defaults.send_eoi = (yyvsp[0].bval);}
+#line 1788 "./ibConfYacc.c"
+    break;
+
+  case 47: /* assign: T_AUTOPOLL  */
+#line 353 "ibConfYacc.y"
+                                          { current_config(parse_arg)->flags |= CN_AUTOPOLL; }
+#line 1794 "./ibConfYacc.c"
+    break;
+
+  case 49: /* assign: T_NAME '=' T_STRING  */
+#line 355 "ibConfYacc.y"
+                                        { strncpy(current_config(parse_arg)->name,(yyvsp[0].sval), sizeof(current_config(parse_arg)->name));}
+#line 1800 "./ibConfYacc.c"
+    break;
+
+  case 50: /* assign: T_MINOR '=' T_NUMBER  */
+#line 356 "ibConfYacc.y"
+                                        { current_config(parse_arg)->defaults.board = (yyvsp[0].ival);}
+#line 1806 "./ibConfYacc.c"
+    break;
+
+  case 51: /* assign: T_TIMO '=' T_TIVAL  */
+#line 357 "ibConfYacc.y"
+                                          { current_config(parse_arg)->defaults.usec_timeout = (yyvsp[0].ival); }
+#line 1812 "./ibConfYacc.c"
+    break;
+
+  case 52: /* assign: T_TIMO '=' T_NUMBER  */
+#line 358 "ibConfYacc.y"
+                                           { current_config(parse_arg)->defaults.usec_timeout = timeout_to_usec((yyvsp[0].ival)); }
+#line 1818 "./ibConfYacc.c"
+    break;
+
   case 56: /* oneflag: T_LLO  */
-#line 266 "ibConfYacc.y"
-                             { current_config( parse_arg )->flags |= CN_SLLO; }
-#line 1729 "./ibConfYacc.c"
+#line 366 "ibConfYacc.y"
+                             { current_config(parse_arg)->flags |= CN_SLLO; }
+#line 1824 "./ibConfYacc.c"
     break;
 
   case 57: /* oneflag: T_DCL  */
-#line 267 "ibConfYacc.y"
-                              { current_config( parse_arg )->flags |= CN_SDCL; }
-#line 1735 "./ibConfYacc.c"
+#line 367 "ibConfYacc.y"
+                              { current_config(parse_arg)->flags |= CN_SDCL; }
+#line 1830 "./ibConfYacc.c"
     break;
 
   case 58: /* oneflag: T_EXCL  */
-#line 268 "ibConfYacc.y"
-                              { current_config( parse_arg )->flags |= CN_EXCLUSIVE; }
-#line 1741 "./ibConfYacc.c"
+#line 368 "ibConfYacc.y"
+                              { current_config(parse_arg)->flags |= CN_EXCLUSIVE; }
+#line 1836 "./ibConfYacc.c"
     break;
 
 
-#line 1745 "./ibConfYacc.c"
+#line 1840 "./ibConfYacc.c"
 
       default: break;
     }
@@ -1939,6 +2034,6 @@ yyreturnlab:
   return yyresult;
 }
 
-#line 271 "ibConfYacc.y"
+#line 371 "ibConfYacc.y"
 
 
